export declare class CSharpRegex {
    private regex;
    private options;
    /**
     * Creates a new CSharpRegex instance.
     * @param {string} pattern The regular expression pattern.
     * @param {string} [options=""] The options to use with the regular expression (e.g., "IgnoreCase", "Multiline").
     */
    constructor(pattern: string, options?: string);
    /**
     * Tests if the pattern matches the input string.
     * @param {string} input The string to test.
     * @returns {boolean} True if the pattern matches the input string, otherwise false.
     */
    IsMatch(input: string): boolean;
    /**
     * Finds the first match of the pattern in the input string.
     * @param {string} input The string to search.
     * @returns {CSharpMatch | null} The first match if found, otherwise null.
     */
    Match(input: string): CSharpMatch | null;
    /**
     * Finds all matches of the pattern in the input string.
     * @param {string} input The string to search.
     * @returns {CSharpMatch[]} An array of all matches.
     */
    Matches(input: string): CSharpMatch[];
    /**
     * Replaces occurrences of the pattern in the input string with the replacement string.
     * @param {string} input The string to perform replacements on.
     * @param {string} replacement The replacement string.
     * @returns {string} The string with replacements made.
     */
    Replace(input: string, replacement: string): string;
    /**
     * Splits the input string into an array of substrings using the pattern as the delimiter.
     * @param {string} input The string to split.
     * @param {number} [count=-1] The maximum number of splits to perform.
     * @param {number} [startat=0] The starting position in the input string.
     * @returns {string[]} An array of substrings.
     */
    Split(input: string, count?: number, startat?: number): string[];
    /**
     * Escapes a string to be used literally in a regex pattern.
     * @param {string} input The string to escape.
     * @returns {string} The escaped string.
     */
    static Escape(input: string): string;
    /**
     * Unescapes a string that was escaped for use in a regex pattern.
     * @param {string} input The string to unescape.
     * @returns {string} The unescaped string.
     */
    static Unescape(input: string): string;
    /**
     * Tests if the pattern matches the input string.
     * @param {string} input The string to test.
     * @param {string} pattern The regular expression pattern.
     * @param {string} [options=""] The options to use with the regular expression.
     * @returns {boolean} True if the pattern matches the input string, otherwise false.
     */
    static IsMatch(input: string, pattern: string, options?: string): boolean;
    /**
     * Finds the first match of the pattern in the input string.
     * @param {string} input The string to search.
     * @param {string} pattern The regular expression pattern.
     * @param {string} [options=""] The options to use with the regular expression.
     * @returns {CSharpMatch | null} The first match if found, otherwise null.
     */
    static Match(input: string, pattern: string, options?: string): CSharpMatch | null;
    /**
     * Finds all matches of the pattern in the input string.
     * @param {string} input The string to search.
     * @param {string} pattern The regular expression pattern.
     * @param {string} [options=""] The options to use with the regular expression.
     * @returns {CSharpMatch[]} An array of all matches.
     */
    static Matches(input: string, pattern: string, options?: string): CSharpMatch[];
    /**
     * Replaces occurrences of the pattern in the input string with the replacement string.
     * @param {string} input The string to perform replacements on.
     * @param {string} pattern The regular expression pattern.
     * @param {string} replacement The replacement string.
     * @param {string} [options=""] The options to use with the regular expression.
     * @returns {string} The string with replacements made.
     */
    static Replace(input: string, pattern: string, replacement: string, options?: string): string;
    /**
     * Splits the input string into an array of substrings using the pattern as the delimiter.
     * @param {string} input The string to split.
     * @param {string} pattern The regular expression pattern.
     * @param {number} [count=-1] The maximum number of splits to perform.
     * @param {number} [startat=0] The starting position in the input string.
     * @param {string} [options=""] The options to use with the regular expression.
     * @returns {string[]} An array of substrings.
     */
    static Split(input: string, pattern: string, count?: number, startat?: number, options?: string): string[];
}
/**
 * A class representing a match found by the CSharpRegex class.
 * @class
 */
declare class CSharpMatch {
    value: string;
    groups: string[];
    index: number;
    /**
     * Creates a new CSharpMatch instance.
     * @param {RegExpMatchArray} match The match array from a regular expression execution.
     */
    constructor(match: RegExpMatchArray);
    /**
     * Returns the next match in the input string.
     * @returns {CSharpMatch | null} The next match if found, otherwise null.
     */
    NextMatch(): CSharpMatch | null;
}
export {};
