/**
 * A class that provides a wrapper around JavaScript's regular expression functionality,
 * designed to handle inline flags similar to Java's regex.  It processes flags within
 * the pattern (e.g., `(?i)` for case-insensitive) and converts them to standard
 * JavaScript RegExp flags.
 * @class
 */
export declare class JavaRegex {
    flags: string;
    pattern: string;
    jsPattern: RegExp;
    /**
     * Creates a new JavaRegex instance.
     * @param {string} pattern The regular expression pattern, potentially including inline flags.
     */
    constructor(pattern: string);
    /**
     * Converts the pattern string, processing inline flags and creating the JavaScript RegExp object.
     * @private
     * @param {string} pattern The input pattern string.
     * @returns {RegExp} A JavaScript RegExp object.
     */
    private convertPattern;
    /**
     * Gets the original pattern string (including inline flags).
     * @returns {string} The original pattern string.
     */
    getPattern(): string;
    /**
     * Tests if a string matches the regular expression.
     * @param {string} str The string to test.
     * @returns {RegExpMatchArray | null} An array containing matching information, or null if no match is found.
     */
    match(str: string): RegExpMatchArray | null;
    /**
     * Replaces all matches in a string with a replacement string.
     * @param {string} str The string to perform replacements on.
     * @param {string} replacement The replacement string.
     * @returns {string} The string with replacements made.
     */
    replace(str: string, replacement: string): string;
    /**
     * Searches a string for a match and returns the index of the match.
     * @param {string} str The string to search.
     * @returns {number} The index of the first match, or -1 if no match is found.
     */
    search(str: string): number;
}
